
!OrderedCollection methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 17:24'!
removeAtIndex: anIndex
	"Remove the element of the collection at position anIndex.  Answer the object removed."

	| obj |
	obj := self at: anIndex.
	self removeIndex: anIndex + firstIndex - 1.
	^obj! !

!Number class methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number between 0 and the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^ DhbMitchellMooreGenerator new floatValue! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
beta: aNumber
		"Computes the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self logBeta: aNumber) exp! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equalsTo: aNumber
		"compare to Float>>closeTo:
		
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	^self relativelyEqualsTo: aNumber upTo: DhbFloatingPointMachine new defaultNumericalPrecision! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorFunction
		"Answer the error function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ DhbErfApproximation new value: self! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
		"Compute the Gamma function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ self > 1
		ifTrue: [ ^ DhbLanczosFormula new gamma: self]
		ifFalse:[ self < 0
						ifTrue: [ Float pi / ( ( Float pi * self) sin * ( 1 - self) gamma)]
						ifFalse:[ ( DhbLanczosFormula new gamma: ( self + 1)) / self]
					]! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logBeta: aNumber
		"Computes the logarithm of the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self logGamma + aNumber logGamma - ( self + aNumber) logGamma! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logGamma
		"Computes the log of the Gamma function (for positive numbers only)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self > 1
		ifTrue: [ DhbLanczosFormula new logGamma: self]
		ifFalse:[ self > 0
						ifTrue: [ ( DhbLanczosFormula new logGamma: ( self + 1) ) - self ln ]
						ifFalse: [ ^self error: 'Argument for the log gamma function must be positive']
					]! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number distributed between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^self class random * self! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
relativelyEqualsTo: aNumber upTo: aSmallNumber
		"compare to Float>>closeTo:
		generally called from Number>>equalsTo:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| norm |
	norm := self abs max: aNumber abs.
	^norm <= DhbFloatingPointMachine new defaultNumericalPrecision
		or: [ (self - aNumber) abs < ( aSmallNumber * norm)]! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Number methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: self! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
at: aRowIndex at: aColumnIndex
		"Answers the aRowIndex-th, aColumnIndex-th entry in the receiver.
		 (c) Copyrights Joseph WHITESELL, 2001, all rights reserved.
		 Initial code: 08/17/2001 "
	^ self rowAt: aRowIndex columnAt: aColumnIndex! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
at: rowIndex at: columnIndex put: value

	self rowAt: rowIndex columnAt: columnIndex put: value
	
! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atColumn: anInteger

	^ self columnAt: anInteger! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atColumn: aColumnIndex put: aCollection

	aCollection withIndexDo: [: value : rowIndex |
		self rowAt: rowIndex columnAt: aColumnIndex put: value ]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atColumn: aColumnNumber put: aCollection startingAt: rowNumber
	" Fill the receiver with aCollection at aColumnNumber beggining
	at rowNumber. "
	
	aCollection withIndexDo: [: value : rowIndex |
		(rowIndex + rowNumber ) <= self numberOfRows 
		ifTrue: 
		[ self rowAt: rowIndex + rowNumber
			columnAt: aColumnNumber 
			put: value ]]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atRow: rowIndex put: aCollection

	aCollection withIndexDo: [: value : columnIndex |
		self rowAt: rowIndex columnAt: columnIndex put: value ]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atRow: rowIndex put: aCollection startingAt: startColumnNumber 
	" Fill the receiver with aCollection at rowIndex beggining at startColumnNumber. "
		
	aCollection withIndexDo: [: value : columnIndex |
		(columnIndex + startColumnNumber ) <= self numberOfColumns 
		ifTrue:
		[ self 
			rowAt: rowIndex 
			columnAt: columnIndex + startColumnNumber
			put: value ]]
	
! !

!DhbMatrix methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeRows: rowsInteger columns: columnsInteger
	"Build empty components for a matrix."
		
	rows := ( 1 to: rowsInteger ) asVector collect: [ :each | DhbVector new: columnsInteger ].! !

!Complex class methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number with abs between 0 and 1." 

	^ self abs: 1.0 random arg: 2 * Float pi random! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99
		
		added to Complex 11 May 2012 Daniel Uber
		 "
	^aPolynomial addNumber: self! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
	"analog to Number>>random. However, the only bound is that the abs of the produced complex is less than the length of the receive. The receiver effectively defines a disc within which the random element can be produced."
	^ self class random * self
	! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Complex methodsFor: '*Math-DHB-NumericalExtensions-complex' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 
		added to Complex class 12 May 2012, Daniel Uber
		"
	^aPolynomial timesNumber: self! !

!Collection methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asVector
	"Convert the receiver to a vector. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector! !

!Quaternion class methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:44'!
random
		"Answers a random quaternion with abs at most one."

	^ (0.5 - Float random) 
		i: (0.5 - Float random)
		j: (0.5 - Float random)
		k: (0.5 - Float random).! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99
		
		added to Quaternion 11 May 2012 Daniel Uber
		 "
	^aPolynomial addNumber: self! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:41'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:41'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:44'!
random
	"analog to Number>>random. The resulting quaternion will have abs at most that of the receiver"
	^ self class random * self.! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:42'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Quaternion methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 6/1/2012 16:42'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 
		added to Complex class 12 May 2012, Daniel Uber
		"
	^aPolynomial timesNumber: self! !

!Integer methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
	self > 0
		ifFalse: [^ self error: 'Attempt to compute the Gamma function of a non-positive integer'].
	^ (self - 1) factorial.! !

!Integer methodsFor: '*Math-DHB-NumericalExtensions' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random integer between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^ DhbMitchellMooreGenerator new integerValue: self! !
